@@ -99,7 +99,7 @@ func jsStart(jobsEnv string) {
 	}
 	JSR, JSW, err = os.Pipe()
 	if err != nil {
-		log.Fatalln(err)
+		log.Fatal(err)
 	}
 	tracef(CJS, "initial fill with %d", jobs)
 	jsTokens[BMakeGoodToken] = int(jobs)
@@ -153,7 +153,7 @@ func jsInit() {
 	func() {
 		defer func() {
 			if err := recover(); err != nil {
-				log.Fatalln(err)
+				log.Fatal(err)
 			}
 		}()
 		JSR = mustParseFd(match[2], "JSR")
