@@ -66,7 +66,7 @@ func statusInit() {
 	var err error
 	r, FdStatus, err = os.Pipe()
 	if err != nil {
-		log.Fatalln(err)
+		log.Fatal(err)
 	}
 	go func() {
 		running := 0
