@@ -44,10 +44,10 @@ func whichdo(tgt *Tgt) (dos []string, err error) {
 			return
 		}
 		for _, ifcreate := range dep.ifcreates {
-			dos = append(dos, ifcreate.RelTo(tgtH))
+			dos = append(dos, ifcreate.RelTo(Cwd))
 		}
 		for _, ifchange := range dep.ifchanges {
-			dos = append(dos, ifchange.tgt.RelTo(tgtH))
+			dos = append(dos, ifchange.tgt.RelTo(Cwd))
 		}
 	}
 	if doFile == "" {
